From 8e8faa9b08cce78245156bb29ad2a4b42857315f Mon Sep 17 00:00:00 2001
From: Wei Shu <weishu@marvell.com>
Date: Tue, 29 Jul 2014 19:01:36 +0800
Subject: [PATCH 3/4] gcore: fix NT_PRSTATUS dump bug in compat mode

the regset data only have the pt_regs data

Signed-off-by: Wei Shu <weishu@marvell.com>
---
 extensions/libgcore/gcore_coredump.c |   22 ++++++++++++----------
 1 file changed, 12 insertions(+), 10 deletions(-)

diff --git a/extensions/libgcore/gcore_coredump.c b/extensions/libgcore/gcore_coredump.c
index 19156d0..d423332 100644
--- a/extensions/libgcore/gcore_coredump.c
+++ b/extensions/libgcore/gcore_coredump.c
@@ -475,6 +475,7 @@ fill_write_thread_core_info(FILE *fp, struct task_context *tc,
         *total += notesize(&memnote);
 	writenote(&memnote, fp, offset);
 	FREEBUF(buf);
+	FREEBUF(memnote.data);
 
         /*
 	 * Fill in the two process-wide notes.
@@ -681,16 +682,13 @@ static void
 fill_prstatus_note(struct elf_note_info *info, struct task_context *tc,
 		   struct memelfnote *memnote)
 {
-	struct elf_prstatus dummy, *prstatus = (struct elf_prstatus *)memnote->data;
+	struct elf_prstatus *prstatus;
 	struct user_regs_struct *regs = (struct user_regs_struct *)memnote->data;
 	ulong pending_signal_sig0, blocked_sig0, real_parent, group_leader,
 		signal, cutime,	cstime;
 
-	/* FIXME: horible workaround during proto-type development... */
-	memset(&dummy, 0, sizeof(dummy));
-	memcpy(&dummy.pr_reg, regs, sizeof(*regs));
-	prstatus = memnote->data;
-	memcpy(prstatus, &dummy, sizeof(dummy));
+	prstatus = (struct elf_prstatus *)GETBUF(sizeof(*prstatus));
+	memcpy(&prstatus->pr_reg, regs, sizeof(*regs));
 
         fill_note(memnote, "CORE", NT_PRSTATUS, sizeof(*prstatus), prstatus);
 
@@ -769,11 +767,15 @@ compat_fill_prstatus_note(struct elf_note_info *info,
 			  struct task_context *tc,
 			  struct memelfnote *memnote)
 {
-	struct compat_elf_prstatus *prstatus =
-		(struct compat_elf_prstatus *)memnote->data;
+	struct compat_elf_prstatus *prstatus;
+	compat_elf_gregset_t *gregset =
+		(compat_elf_gregset_t *)memnote->data;
 	ulong pending_signal_sig0, blocked_sig0, real_parent, group_leader,
 		signal, cutime,	cstime;
 
+	prstatus = (struct compat_elf_prstatus *)GETBUF(sizeof(*prstatus));
+	memcpy(&prstatus->pr_reg, gregset, sizeof(*gregset));
+
         fill_note(memnote, "CORE", NT_PRSTATUS, sizeof(*prstatus), prstatus);
 
         /* The type of (sig[0]) is unsigned long. */
@@ -918,8 +920,8 @@ static int uvtop_quiet(ulong vaddr, physaddr_t *paddr)
 	 * in the last line showing 0x1c08000 is correct one.
 	 *
 	 * crash> vtop 0xffffffffff600000
-	 * VIRTUAL           PHYSICAL        
-	 * ffffffffff600000  7f600000        
+	 * VIRTUAL           PHYSICAL
+	 * ffffffffff600000  7f600000
 	 *
 	 * PML4 DIRECTORY: ffffffff81a85000
 	 * PAGE DIRECTORY: 1a87067
-- 
1.7.9.5

